"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bindings_1 = require("@mavenomics/bindings");
const table_1 = require("@mavenomics/table");
const CodeMirror = require("codemirror");
const jshint_js_1 = require("jshint/dist/jshint.js");
if (window)
    window.JSHINT = jshint_js_1.JSHINT;
require("codemirror/addon/lint/javascript-lint");
require("./javascript-hint");
//Todo: InputWidgets.Code should use a service to generate the codemirror config.
//That way the following code can be registered with the service instead of using this InitHook hack.
//Essentially making InputWidgets.Code pluginable.
function getJavaScriptReferencedGlobals(text) {
    const comment = bindings_1.getCommentLine(text, null);
    return comment != null ? bindings_1.getGlobalsFromComment(comment) : null;
}
//Todo: Subscribe to the globals service and only update when globals are added/removed.
function updateGlobalsCompletion(cm) {
    const options = cm.options;
    if (options) {
        let hintOpts = options.hintOptions;
        if (hintOpts && hintOpts.globalScope && hintOpts.additionalContext) {
            const globalsService = options && options.context ? options.context.globals : null;
            const allGlobals = globalsService ? Array.from(globalsService).map((g) => g.name) : [];
            //Try to find explicitly referenced globals.
            //If there are no explicitly referenced globals then we show all globals.
            const globals = getJavaScriptReferencedGlobals(cm.getDoc().getValue()) || allGlobals;
            let globalsObj = globals.reduce((o, n) => (o[n] = "", o), {});
            hintOpts.globalScope.globals = globalsObj;
            hintOpts.additionalContext.globals = globalsObj;
        }
    }
}
CodeMirror.defineInitHook((cm) => {
    CodeMirror.on(cm, "optionChange", (cm, option) => {
        if (option === "mode") {
            if (cm.getOption("mode") === "text/x-jsmql" && cm.options && cm.options.hintOptions) {
                cm.options.hintOptions.esversion = 9; //Enable es9. This is for await and mql templates support
                //This gives us autocomplete for TableHelper.<x>
                cm.options.hintOptions.additionalContext = { TableHelper: table_1.TableHelper };
                //Copy window so that we can add globals
                let windowClone = {};
                for (let key of Reflect.ownKeys(window)) {
                    windowClone[key] = window[key];
                }
                //Add TableHelper, RunMql, etc
                cm.options.hintOptions.globalScope = Object.assign({}, windowClone, {
                    TableHelper: table_1.TableHelper,
                    RunMql: function () { },
                    mql: "",
                    expr: function () { },
                    ImportScriptFromUrl: function () { },
                });
                updateGlobalsCompletion(cm);
                cm.on("change", updateGlobalsCompletion);
            }
            else {
                cm.off("change", updateGlobalsCompletion);
            }
        }
    });
});
